/*******************************************************************************
 * Copyright (c) 2007 Red Hat, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Red Hat - initial API and implementation
 *******************************************************************************/

package org.eclipse.linuxtools.rpm.ui.editor.parser;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.linuxtools.rpm.ui.editor.SpecfileLog;

public class SpecfileSource extends SpecfileElement {
	int number;
	int lineNumber = -1;
	String fileName;
	static final int SOURCE = 0;
	static final int PATCH = 1;
	int sourceType;
	List<Integer> linesUsed;
	
	public int getSourceType() {
		return sourceType;
	}
	public void setSourceType(int sourceType) {
		this.sourceType = sourceType;
	}
	public SpecfileSource(int number, String fileName) {
		super("source");
		this.number = number;
		this.fileName = fileName;
		this.linesUsed = new ArrayList<Integer>();
	}
	public String getFileName() {
		return resolve(fileName);
	}
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	public int getNumber() {
		return number;
	}
	public void setNumber(int number) {
		this.number = number;
	}
	public void addLineUsed(int lineNumber) {
		linesUsed.add(new Integer(lineNumber));
	}
	public void removeLineUsed(int lineNumber) {
		linesUsed.remove(new Integer(lineNumber));
	}
	public List<Integer> getLinesUsed() {
		return linesUsed;
	}
	public String toString() {
		if (sourceType == SOURCE)
			return "Source #" + number + " (line #" + lineNumber + ", used on lines " + getLinesUsed() + ") -> " + fileName;
		return "Patch #" + number + " (line #" + lineNumber + ", used on lines " + getLinesUsed() + ") -> " + fileName;
	}
	
	// Note that changeReferences assumes that the number of the source/patch
	// has *already been set*.  If this is not true, it will simply do nothing
	public void changeReferences(int oldPatchNumber) {
		Specfile specfile = this.getSpecfile();
		Pattern patchPattern;
		if (oldPatchNumber == 0) {
			patchPattern = Pattern.compile("%patch" + oldPatchNumber + "|%patch");
		} else {
			patchPattern = Pattern.compile("%patch" + oldPatchNumber);
		}
		for (int lineNumber: getLinesUsed()){
			String line;
			try {
				line = specfile.getLine(lineNumber);
				Matcher patchMatcher = patchPattern.matcher(line);
				if (!patchMatcher.find()) {
					System.out.println("error:  can't match " + patchPattern.pattern());
//					throw new BadLocationException("can't match " + patchPattern);
				}
				specfile.changeLine(lineNumber, line.replaceAll(patchPattern.pattern(), "%patch" + number));
			} catch (BadLocationException e) {
				SpecfileLog.logError(e);
			}
		}
	}
	public void changeDeclaration(int oldPatchNumber) {
		Specfile specfile = this.getSpecfile();
		Pattern patchPattern;
		if (oldPatchNumber == 0) {
			patchPattern = Pattern.compile("Patch" + oldPatchNumber + "|Patch");
		} else {
			patchPattern = Pattern.compile("Patch" + oldPatchNumber);
		}
		String line;
		try {
			line = specfile.getLine(lineNumber);
			Matcher patchMatcher = patchPattern.matcher(line);
			if (!patchMatcher.find())
				// TODO: Maybe we can throw a exception here.
				System.out.println("error: can't match " + patchPattern.pattern());
			specfile.changeLine(lineNumber, line.replaceAll(patchPattern.pattern(), "Patch" + number));
		} catch (BadLocationException e) {
			SpecfileLog.logError(e);
		}
	}
	public int getLineNumber() {
		return lineNumber;
	}
	public void setLineNumber(int lineNumber) {
		this.lineNumber = lineNumber;
	}
}
